//////////////////////////////////////////////////////////////////////////////
//   ///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "_DRAW_3D.h"
#include "_3D.h"

//////////////////////////////////////////////////////////////////////////////
//   /////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void Falloff(int f)
{ 
  DrawRect(RegionFull(),RGBis(f,f,f),modeMUL);
};

//////////////////////////////////////////////////////////////////////////////
inline tfloat dist(tfloat x1, tfloat y1, tfloat x2, tfloat y2)
{
  x2 = x2 - x1; y2 = y2 - y1;
  return sqrt(x2 * x2 + y2 * y2);
};

//////////////////////////////////////////////////////////////////////////////
inline dwRGB DIColor(dwRGB c, int m = 255)
{ 
  return RGBmultV(c, DrawIntensityI * m / 255);
};

//////////////////////////////////////////////////////////////////////////////
//   //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
tfloat Hermitit(tfloat t)
{
  t = 1 - t;
  return t * t * (3 - 2 * t);
};

tfloat Bublik(tfloat t)
{
  if (t <= 0.5) t = t * 2; else t = 1 - (t - 0.5) * 2;
  return t * t * (3 - 2 * t);
};


//////////////////////////////////////////////////////////////////////////////
// () GArray //////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
class GArray
{ public:
  int xD, yD, vFilter, vColor, Mode, Painted; 
  tfloat vMin, vMax, vDelta, vRadL, vRandom, vRadDrawLim, Force, ForceInc;
  tfloat_arr *Value;
  dwRGB ColorD, ColorL;

  tfloat (*EPaintFunc)(tfloat);
  
  void filterUp()                             //  : 
  {
    for (int y = 0; y < yD - 1; y++)          //      
    for (int x = 0; x < xD; x++)(*Value)[y * xD + x] = (*Value)[(y + 1) * xD + x] + fRandom(-vRandom, vRandom); 
  };

  void filterDown()                           //  : 
  {
    for (int y = yD - 1; y > 1; y--)
    for (int x = 0; x < xD; x++)(*Value)[y * xD + x] = (*Value)[(y - 1) * xD + x] + fRandom(-vRandom, vRandom); 
  };

  void filterRight()                           //  : 
  {
    for (int y = 0; y < yD; y++) 
    for (int x = xD - 1; x > 1; x--)(*Value)[y * xD + x] = (*Value)[y * xD + x - 1] + fRandom(-vRandom, vRandom) ;  
  };

  void filterLeft()                           //  : 
  {
    for (int y = 0; y < yD; y++)
    for (int x = 0; x < xD - 1; x++)(*Value)[y * xD + x] = (*Value)[y * xD + x + 1] + fRandom(-vRandom, vRandom);
  };

  void Filter()                              //   []
  {
    switch(vFilter)
    { default: break;
      case 1: filterUp(); break;
      case 2: filterDown(); break;
      case 3: filterRight(); break;
      case 4: filterLeft(); break;
    };
  };

  void SetColor(int c)
  { vColor = c;
    switch(c)
    { case 0: ColorD = RGBis(0, 50, 170); ColorL = RGBis(15, 155, 255); break;
      case 1: ColorD = RGBis(105, 0, 0);  ColorL = RGBis(255, 150, 0);  break;
      case 2: ColorD = RGBis(35, 65, 0);  ColorL = RGBis(150, 215, 0);  break;
    };
  };

  void SetFilter(int f)  { vFilter = f; };

  void SetMinMax(tfloat mn, tfloat mx) { vMin = mn; vMax = mx; vDelta = vMax- vMin; };

  void Create()                              //  
  {
    xD = DrawXR / 8; yD = DrawYR  / 8;                      //    -
    Value = (tfloat_arr *)malloc(xD * yD * sizeof(tfloat)); //                  
    for (int i = 0; i < xD * yD; i++)(*Value)[i] = vMin;   //    
  };

  void Done() { free(Value); Value = NULL; };

  GArray()                                   // 
  {
    Mode = modeADD; vFilter = 0; SetColor(0); SetFilter(1); SetMinMax(0.25, 4);
    vRadL = 1.25; vRadDrawLim = 0; Value = NULL; Force = 0; vRandom = 0.15; Painted = no; 
    ForceInc = 0.0025; EPaintFunc = Hermitit;
  };

  ~GArray() { Done(); };                     //  

  void Paint(int x, int y, int rad, int sm = Yes)     // 
  { 
    if (sm) floatLinc(Force, 1, ForceInc); else Force = 1;
    tfloat r;
    int xx = (x - 4) / 8,           yy = (y - 4) / 8,
        x1 = max(xx - rad, 0),      y1 = max(yy - rad, 0),
        x2 = min(xx + rad, xD - 1), y2 = min(yy + rad, yD - 1); 
    for (int y = y1; y <= y2; y++)
    for (int x = x1; x <= x2; x++)
    { r = dist(xx, yy, x, y); 
      if (r < rad) floatLinc((*Value)[y * xD + x], vMax, EPaintFunc(r / rad) * Force);
    };  
    Painted = Yes;
  };

  void Draw()                                // 
  {
    tfloat *r;
    for (int y = 0; y < yD; y++)
    for (int x = 0; x < xD; x++)
    { r = &(*Value)[y * xD + x];
      *r = max(*r * 0.97 - 0.005, vMin);
      if (*r > vRadDrawLim)
      {  DrawCircle(4 + x * 8, 4 + y * 8, *r, DIColor(RGBblend(ColorD, ColorL,(*r - vMin) / vDelta * 255 + 0.5)), Mode); // :    
        if (*r > vRadL + 0.5) DrawCircle(4 + x * 8, 4 + y * 8, *r - (vMax - vRadL), DIColor(ColorL, 175), modeADD);
      };
    };  
    if (!Painted) Force = max(Force * 0.9 - 0.0025, 0);
    Painted = no;
  };
};

//////////////////////////////////////////////////////////////////////////////
//    /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
GArray garray;

//////////////////////////////////////////////////////////////////////////////////////////////////
int MousePressNow = no;

void mouseLBDown()                           //  []  
{ 
  MousePressNow = Yes;
};

void mouseLBUp()                           //  []  
{ 
  MousePressNow = no;
};


void mouseMoved()                           //  []  
{ 
  if (!MousePressNow) return;
  garray.Paint(II.Mouse.X, II.Mouse.Y, 19, II.KeyDown[_Shift_]);
};

//////////////////////////////////////////////////////////////////////////////////////////////////
int keyExit = 0, HelpTimer = 0;

void keyPressed()                           //  [ ] 
{
  if (II.KeyUp[_Quote_]) garray.SetColor((3 + garray.vColor - 1) % 3);  
  if (II.KeyUp[_Enter_]) garray.SetColor((garray.vColor + 1) % 3);
  if (II.KeyUp[_CU_]) garray.SetFilter(1);
  if (II.KeyUp[_CD_]) garray.SetFilter(2); 
  if (II.KeyUp[_CR_]) garray.SetFilter(3); 
  if (II.KeyUp[_CL_]) garray.SetFilter(4);
  if (II.KeyUp[_Space_]) garray.SetFilter(0);
  if (II.KeyUp[_Del_]) if (garray.vRandom > 0.001) garray.vRandom = 0; else garray.vRandom = 0.15; 
  if (II.KeyUp[_F1_]) if (HelpTimer) HelpTimer = 0; else HelpTimer = 25 * 10;
  keyExit = (II.KeyUp[_Esc_]);
  II.Reset();
};


//////////////////////////////////////////////////////////////////////////////
//     /////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
TextSPR *info;

void CommonsInit()
{ RandomSeed();

  FC_FpsSet(AUTO);            //    
  FC_DrawPaint = Yes;         //  fps   DrawPaint();
  
  DrawPspMax = 5;              //    
  DrawSetPspClipPlanes(-115); //    , ,     
                               //   ,       
  //DrawSetFogPlanes(500,-115,effSEXPONENT); //     .

  Draw3D_Init();
  ForceActivate();
  ScreenLoadWait();

  info = new TextSPR(L"TYPE_EN_RU");

  garray.Create(); //garray.vRadDrawLim = 0.5;

  OnMouseMoveDo(mouseMoved);   //     
  OnMouseLButtonDownDo(mouseLBDown);   //     
  OnMouseLButtonUpDo(mouseLBUp);   //     
  OnKeyUpDo(keyPressed);       //     
  keyPressed();                //     ...
};


//////////////////////////////////////////////////////////////////////////////////////////////////
void CommonsDone()
{
  delete info;
  Draw3D_Done();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
void DrawInfo()
{
  tRegion r = RegionCenterIs(208, 240);
  DrawRegionStore.Save(r);
  DrawRect(r, RGBis(50, 50, 50), modeMUL); 
  info->Place(0, 0, modeADD + fromCC, "Conrol Keys:\n\n\nQutes, Enter - change Color\n...............................................................................\n\nCursor - set Move Direction\n\nSpace - set No Move\n\nDel - set On/Off Randomize\n...............................................................................\n\nF1 - hide/show Help\n\nEsc - Exit\n...............................................................................\n\nAnd... - Mouse Click & Motion"); info->Draw();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
void DrawAll()
{
  if (HelpTimer) DrawInfo();
  DrawPaint();
  if (HelpTimer) { DrawRegionStore.Restore();  HelpTimer--; };
};

//////////////////////////////////////////////////////////////////////////////
//   //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void Main()
{
  CommonsInit();                  //   ;  

  SetDrawIntensity(0.4);            //      
  //DrawScaleSet(1);                // 

  //garray.EPaintFunc = Bublik;
  garray.SetFilter(0);

  while (!keyExit)
   { Falloff(160);
     garray.Draw(); garray.Filter();          // 
     DrawAll();
   };   

  CommonsDone();                  //   ;
};
//////////////////////////////////////////////////////////////////////////////
int LetProcess()
{
  WinName = L"CG : Array";      //    
  VS_CColor = RGBis(0,0,0);    //   
  DrawSet(FullScreen);         //   ( )  , FullScreen   
  //SetDraw(640,480);
  ProcessFunc = Main;           //    -  
  return on;
};
int Init_LetProcess = LetProcess();

